use super::Route;
use crate::{AppState, admin::station::Station, err, page, succ};
use axum::{
    Form, Router,
    extract::State,
    response::{IntoResponse, Redirect},
    routing::{get, post},
};
use maud::{Markup, PreEscaped, html};
use serde::Deserialize;
use sqlx::SqlitePool;
use std::sync::Arc;
use tower_sessions::Session;

async fn index(State(db): State<Arc<SqlitePool>>, session: Session) -> Markup {
    let routes = Route::all(&db).await;

    let content = html! {
        h1 {
            a href="/admin" { "↩️" }
            "Routen"
        }
        article {
            em { "Routen " }
            "definieren welche "
            a href="/admin/station" { "Stationen" }
            " von den "
            a href="/admin/team" { "Teams" }
            " in welcher Reihenfolge abgeklappert werden sollen. Wenn es verschiedene Kategorien (zB Kinder- und Erwachsenenwertung) gibt, kannst du auch mehrere Routen mit (teils) überlappenden Stationen erstellen."
        }
        ol {
            @for route in &routes{
                li {
                    @if route.stations(&db).await.is_empty() {
                        em data-tooltip="Keine Stationen zugeteilt" {
                            "⚠️"
                        }
                    }
                     a href=(format!("/admin/route/{}", route.id)){
                        (route.name)
                     }
                     a href=(format!("/admin/route/{}/delete", route.id))
                       onclick="return confirm('Bist du sicher, dass die Route gelöscht werden soll? Das kann _NICHT_ mehr rückgängig gemacht werden.');" {
                         "🗑️"
                     }
                }
            }
        }
        @if routes.is_empty() {
            article class="warning" {
                "Es gibt noch keine Routen. Das kannst du hier ändern ⤵️"
            }
        }
        h2 { "Neue Route" }
        form action="/admin/route" method="post" {
            fieldset role="group" {
                input type="text" name="name" placeholder="Routenname" required;
                input type="submit" value="Neue Route";
            }
        }
    };
    page(content, session, false).await
}

#[derive(Deserialize)]
struct CreateForm {
    name: String,
}

async fn create(
    State(db): State<Arc<SqlitePool>>,
    session: Session,
    Form(form): Form<CreateForm>,
) -> impl IntoResponse {
    match Route::create(&db, &form.name).await {
        Ok(()) => succ!(session, "Route '{}' erfolgreich erstellt!", form.name),
        Err(e) => err!(
            session,
            "Route '{}' konnte _NICHT_ erstellt werden, da es bereits eine Route mit diesem Namen gibt ({e})!",
            form.name
        ),
    }

    Redirect::to("/admin/route")
}

async fn delete(
    State(db): State<Arc<SqlitePool>>,
    session: Session,
    axum::extract::Path(id): axum::extract::Path<i64>,
) -> impl IntoResponse {
    let Some(route) = Route::find_by_id(&db, id).await else {
        err!(
            session,
            "Route mit ID {id} konnte nicht gelöscht werden, da sie nicht existiert"
        );

        return Redirect::to("/admin/route");
    };

    match route.delete(&db).await {
        Ok(()) => succ!(session, "Route '{}' erfolgreich gelöscht!", route.name),
        Err(e) => err!(
            session,
            "Route '{}' kann nicht gelöscht werden, da sie bereits verwendet wird. ({e})",
            route.name
        ),
    }

    Redirect::to("/admin/route")
}

async fn view(
    State(db): State<Arc<SqlitePool>>,
    session: Session,
    axum::extract::Path(id): axum::extract::Path<i64>,
) -> Result<Markup, impl IntoResponse> {
    let Some(route) = Route::find_by_id(&db, id).await else {
        err!(
            session,
            "Route mit ID {id} konnte nicht geöffnet werden, da sie nicht existiert"
        );

        return Err(Redirect::to("/admin/route"));
    };

    let cur_stations = route.stations(&db).await;
    let stations_not_in_route = route.stations_not_in_route(&db).await;

    let teams = route.teams(&db).await;

    let content = html! {
        h1 {
            a href="/admin/route" { "↩️" }
            "Route " (route.name)
        }
        article {
            details {
                summary { "Routenname bearbeiten ✏️" }
                form action=(format!("/admin/route/{}/name", route.id)) method="post" {
                    input type="text" name="name" value=(route.name) required;
                    input type="submit" value="Speichern";
                }
            }
        }
        h2 { "Stationen" }
        @if cur_stations.is_empty() {
            @if stations_not_in_route.is_empty() {
                article class="error" {
                    (PreEscaped("Bevor du einer Route Stationen zuweisen kannst, musst du die Stationen erstellen &rarr; "))
                    a role="button" href="/admin/station" {
                        "Station erstellen"
                    }
                }
            } @else {
                article class="warning" {
                    "Diese Route hat noch keine Stationen zugewiesen. Das kannst du hier ändern ⤵️"
                }
            }
        } @else {
            ol {
                @for (idx, station) in cur_stations.into_iter().enumerate() {
                    li {
                        (station.name)
                         @if idx > 0 {
                            a href=(format!("/admin/route/{}/move-station-higher/{}", route.id, station.id)){
                                em data-tooltip=(format!("{} nach vor reihen", station.name)) {
                                    "⬆️"
                                }
                            }
                         }
                        a href=(format!("/admin/route/{}/delete-station/{}", route.id, station.id))
                          onclick="return confirm('Bist du sicher, dass die Station von der Route entfernt werden soll?');" {
                            "🗑️"
                        }
                    }
                }
            }
        }
        @if !stations_not_in_route.is_empty(){
            form action=(format!("/admin/route/{}/add-station", route.id)) method="post" {
                select name="station" aria-label="Hinzuzufügende Station auswählen" required {
                    @for station in &stations_not_in_route {
                        option value=(station.id) {
                            (station.name)
                        }
                    }
                    input type="submit" value="Hinzufügen";
                }
        }
        }
        h2 { "Teams mit dieser Route" }
        @if teams.is_empty() {
            article {
                "Noch keine Team ist dieser Route zugeteilt."
                a role="button" href="/admin/team" {
                    "Zu den Teams"
                }

            }
        } @else {
            ol {
                @for team in &teams {
                    li {
                        a href=(format!("/admin/team/{}", team.id)) {
                            (team.name)
                        }
                    }
                }
            }
        }

    };
    Ok(page(content, session, false).await)
}

#[derive(Deserialize)]
struct UpdateNameForm {
    name: String,
}
async fn update_name(
    State(db): State<Arc<SqlitePool>>,
    session: Session,
    axum::extract::Path(id): axum::extract::Path<i64>,
    Form(form): Form<UpdateNameForm>,
) -> impl IntoResponse {
    let Some(route) = Route::find_by_id(&db, id).await else {
        err!(
            session,
            "Route mit ID {id} konnte nicht geöffnet werden, da sie nicht existiert"
        );

        return Redirect::to("/admin/route");
    };

    route.update_name(&db, &form.name).await;

    succ!(
        session,
        "Route '{}' heißt ab sofort {}.",
        route.name,
        form.name
    );

    Redirect::to(&format!("/admin/route/{id}"))
}

#[derive(Deserialize)]
struct AddStationForm {
    station: i64,
}
async fn add_station(
    State(db): State<Arc<SqlitePool>>,
    session: Session,
    axum::extract::Path(id): axum::extract::Path<i64>,
    Form(form): Form<AddStationForm>,
) -> impl IntoResponse {
    let Some(route) = Route::find_by_id(&db, id).await else {
        err!(
            session,
            "Route mit ID {id} konnte nicht geöffnet werden, da sie nicht existiert"
        );

        return Redirect::to("/admin/route");
    };
    let Some(station) = Station::find_by_id(&db, form.station).await else {
        err!(
            session,
            "Station mit ID {id} konnte nicht hinzugefügt werden, da sie nicht existiert"
        );

        return Redirect::to(&format!("/admin/route/{id}"));
    };

    match route.add_station(&db, &station).await {
        Ok(()) => succ!(
            session,
            "Station {} wurde der Route {} hinzugefügt",
            station.name,
            route.name
        ),
        Err(e) => err!(
            session,
            "Station {} kann nur 1x der Route {} hinzugefügt werden. ({e})",
            station.name,
            route.name
        ),
    }

    Redirect::to(&format!("/admin/route/{id}"))
}

async fn delete_station(
    State(db): State<Arc<SqlitePool>>,
    session: Session,
    axum::extract::Path((route_id, station_id)): axum::extract::Path<(i64, i64)>,
) -> impl IntoResponse {
    let Some(route) = Route::find_by_id(&db, route_id).await else {
        err!(
            session,
            "Konnte keine Station von Route mit ID {route_id} entfernen, da diese Route nicht existiert."
        );

        return Redirect::to("/admin/route");
    };
    let Some(station) = Station::find_by_id(&db, station_id).await else {
        err!(
            session,
            "Konnte Station mit der ID {station_id} nicht von der Route {} entfernen, da die Station nicht existiert.",
            route.name
        );

        return Redirect::to(&format!("/admin/route/{route_id}"));
    };

    if route.delete_station(&db, &station).await {
        succ!(
            session,
            "Station '{}' wurde von der Route '{}' gelöscht",
            station.name,
            route.name
        );
    } else {
        err!(
            session,
            "Station '{}' konnte nicht von der Route '{}' gelöscht werden, da diese nicht auf dieser Route liegt",
            station.name,
            route.name
        );
    }

    Redirect::to(&format!("/admin/route/{route_id}"))
}

async fn move_station_higher(
    State(db): State<Arc<SqlitePool>>,
    session: Session,
    axum::extract::Path((route_id, station_id)): axum::extract::Path<(i64, i64)>,
) -> impl IntoResponse {
    let Some(route) = Route::find_by_id(&db, route_id).await else {
        err!(
            session,
            "Konnte keine Station von Route mit ID {route_id} verschieben, da diese Route nicht existiert."
        );

        return Redirect::to("/admin/route");
    };
    let Some(station) = Station::find_by_id(&db, station_id).await else {
        err!(
            session,
            "Konnte Station mit der ID {station_id} nicht in der Route {} verschieben, da die Station nicht existiert.",
            route.name
        );

        return Redirect::to(&format!("/admin/route/{route_id}"));
    };

    if route.move_station_higher(&db, &station).await {
        succ!(
            session,
            "Station '{}' wurde in der Route '{}' erfolgreich vorgereiht",
            station.name,
            route.name
        );
    } else {
        err!(
            session,
            "Station '{}' konnte in der Route '{}' nicht vorgereiht werden, da diese nicht auf dieser Route liegt",
            station.name,
            route.name
        );
    }

    Redirect::to(&format!("/admin/route/{route_id}"))
}

pub(super) fn routes() -> Router<AppState> {
    Router::new()
        .route("/", get(index))
        .route("/", post(create))
        .route("/{id}/delete", get(delete))
        .route("/{id}", get(view))
        .route("/{id}/name", post(update_name))
        .route("/{id}/add-station", post(add_station))
        .route(
            "/{route_id}/delete-station/{station_id}",
            get(delete_station),
        )
        .route(
            "/{route_id}/move-station-higher/{station_id}",
            get(move_station_higher),
        )
}
