use crate::{err, page, succ, AppState};
use async_trait::async_trait;
use axum::{
    http::StatusCode,
    response::{IntoResponse, Redirect},
    routing::{get, post},
    Form, Router,
};
use axum_login::{AuthUser, AuthnBackend};
use maud::{html, Markup};
use password_auth::verify_password;
use serde::{Deserialize, Serialize};
use sqlx::{FromRow, SqlitePool};
use tower_sessions::Session;

pub type UserId<Backend> = <<Backend as AuthnBackend>::User as AuthUser>::Id;

#[derive(Clone, Serialize, Deserialize, FromRow, Debug)]
pub struct User {
    pub(crate) id: i64,
    pub(crate) name: String,
    pub(crate) pw: String,
    pub(crate) require_new_password_code: Option<String>,
}

#[derive(Debug, Clone, Deserialize)]
pub struct Credentials {
    pub name: String,
    pub password: String,
}

impl AuthUser for User {
    type Id = i64;

    fn id(&self) -> Self::Id {
        self.id
    }

    fn session_auth_hash(&self) -> &[u8] {
        self.pw.as_bytes()
    }
}

#[derive(Debug, Clone)]
pub struct Backend {
    db: SqlitePool,
}

impl Backend {
    pub(crate) fn new(db: SqlitePool) -> Self {
        Self { db }
    }
}

#[derive(Debug, thiserror::Error)]
pub enum Error {
    #[error(transparent)]
    Sqlx(#[from] sqlx::Error),
}

#[async_trait]
impl AuthnBackend for Backend {
    type User = User;
    type Credentials = Credentials;
    type Error = Error;

    async fn authenticate(
        &self,
        creds: Self::Credentials,
    ) -> Result<Option<Self::User>, Self::Error> {
        let user: Option<Self::User> = sqlx::query_as(
            "SELECT id, name, pw, require_new_password_code FROM user WHERE name = ? ",
        )
        .bind(creds.name)
        .fetch_optional(&self.db)
        .await?;

        // We're using password-based authentication--this works by comparing our form
        // input with an argon2 password hash.
        Ok(user.filter(|user| {
            verify_password(creds.password, &user.pw).is_ok()
                && user.require_new_password_code.is_none()
        }))
    }

    async fn get_user(&self, user_id: &UserId<Self>) -> Result<Option<Self::User>, Self::Error> {
        let user =
            sqlx::query_as("SELECT id, name, pw, require_new_password_code FROM user WHERE id = ?")
                .bind(user_id)
                .fetch_optional(&self.db)
                .await?;

        Ok(user)
    }
}

pub type AuthSession = axum_login::AuthSession<Backend>;

pub fn routes() -> Router<AppState> {
    Router::new()
        .route("/login", get(self::login))
        .route("/login", post(self::login_post))
        .route("/logout", get(self::logout))
}

async fn login(session: Session) -> Markup {
    let content = html! {
        h1 { "Login" }
        form action="/auth/login" method="post" {
            label  {
                "Name"
                input type="text" name="name";
            }
            label {
                "Passwort"
                input type="password" name="password";
            }
            input type="submit" value="Einloggen";
        }
    };

    // TODO: generate okayish looking login page

    page(content, session, false).await
}

pub async fn login_post(
    mut auth_session: AuthSession,
    session: Session,
    Form(creds): Form<Credentials>,
) -> impl IntoResponse {
    let user = match auth_session.authenticate(creds.clone()).await {
        Ok(Some(user)) => user,
        Ok(None) => {
            err!(session, "Invalid credentials");

            return Redirect::to("/auth/login").into_response();
        }
        Err(_) => return StatusCode::INTERNAL_SERVER_ERROR.into_response(),
    };

    if auth_session.login(&user).await.is_err() {
        return StatusCode::INTERNAL_SERVER_ERROR.into_response();
    }

    succ!(session, "Successfully logged in as {}", user.name);

    Redirect::to("/admin").into_response()
}

pub async fn logout(mut auth_session: AuthSession) -> impl IntoResponse {
    match auth_session.logout().await {
        Ok(_) => Redirect::to("/auth/login").into_response(),
        Err(_) => StatusCode::INTERNAL_SERVER_ERROR.into_response(),
    }
}
